/**
 * @file
 *
 * @brief openMTPK ARITHMETIC MODULE 
 * 
 * @details This file serves as the core Arithmetic module housing the 
 * basic operations and operations based on/using arithmetic. 
 */
#ifndef LINALG_HPP
#define LINALG_HPP


/**
 * @brief \b Arithmetic \b Basics
 * @details Basics of Arithmetic 
 */
#define ARITHMETIC_BASICS
#include "arithmetic/arith_base.hpp"
/*****************************************************************************/

/**
 * @brief \b Arithmetic \b Operations
 * @details Encompasses Arithmetic related operations
 */
#define ARITHMETIC_OPERATIONS
#include "arithmetic/arith_ops.hpp"
/*****************************************************************************/

#endif

